package cn.com.qmth.examcloud.web.filestorage.impl;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.web.filestorage.FileStorage;
import cn.com.qmth.examcloud.web.filestorage.FileStorageHelper;
import cn.com.qmth.examcloud.web.filestorage.FileStoragePathEnvInfo;
import cn.com.qmth.examcloud.web.filestorage.FileStorageType;
import cn.com.qmth.examcloud.web.filestorage.YunHttpRequest;
import cn.com.qmth.examcloud.web.filestorage.YunPathInfo;
import cn.com.qmth.examcloud.web.upyun.UpYunClient;
import cn.com.qmth.examcloud.web.upyun.UpYunPathInfo;
import cn.com.qmth.examcloud.web.upyun.UpyunPathEnvironmentInfo;
import cn.com.qmth.examcloud.web.upyun.UpyunService;
import cn.com.qmth.examcloud.web.upyun.UpyunSite;
import cn.com.qmth.examcloud.web.upyun.UpyunSiteManager;

@Service(value = "upyunFileStorage")
public class UpyunFileStorageImpl implements FileStorage {
	@Autowired
	private UpyunService upyunService;

	@Override
	public YunPathInfo saveFile(File file, String path) {
		String siteId="transPath";
		FileStoragePathEnvInfo env = new FileStoragePathEnvInfo();
		env.setRelativePath(path);
		return saveFile(siteId, env, file, null);
	}

	@Override
	public String realPath(String path) {
		String upyunId=FileStorageHelper.getYunId(path);
		String urlpath=FileStorageHelper.getPath(path);
		UpYunClient c=UpyunSiteManager.getUpYunClientByUpyunId(upyunId);
		return FileStorageHelper.getUrl(c.getDomain(), urlpath);
	}

	@Override
	public YunPathInfo saveFile(String siteId, FileStoragePathEnvInfo env, File file,String md5) {
		InputStream in=null;
		try {
			in = new FileInputStream(file);
			return saveFile(siteId, env, in, md5);
		} catch (FileNotFoundException e) {
			throw new StatusException("1001", "上传出错", e);
		} finally {
			if(in!=null) {
				try {
					in.close();
				} catch (IOException e) {
				}
			}
		}
	}
	
	private UpyunPathEnvironmentInfo of(FileStoragePathEnvInfo env) {
		UpyunPathEnvironmentInfo ret=new UpyunPathEnvironmentInfo();
		ret.setExt1(env.getExt1());
		ret.setExt2(env.getExt2());
		ret.setExt3(env.getExt3());
		ret.setExt4(env.getExt4());
		ret.setExt5(env.getExt5());
		ret.setFileSuffix(env.getFileSuffix());
		ret.setRelativePath(env.getRelativePath());
		ret.setRootOrgDomain(env.getRootOrgDomain());
		ret.setRootOrgId(env.getRootOrgId());
		ret.setTimeMillis(env.getTimeMillis());
		ret.setUserId(env.getUserId());
		return ret;
	}

	@Override
	public YunHttpRequest getSignature(String siteId, FileStoragePathEnvInfo env, String md5) {
		YunHttpRequest req=upyunService.buildUpYunHttpRequest(siteId, of(env), md5);
		return req;
	}

	@Override
	public void deleteFile(String path) {
		String upyunId=FileStorageHelper.getYunId(path);
		String urlpath=FileStorageHelper.getPath(path);
		upyunService.deleteByUpyunId(upyunId, urlpath);
	}

	@Override
	public YunPathInfo saveFile(String siteId, FileStoragePathEnvInfo env, InputStream in, String md5) {
		UpYunPathInfo pathInfo =upyunService.writeFile(siteId, of(env), in, md5);
		UpyunSite site=UpyunSiteManager.getUpyunSite(siteId);
		String relativePath=getTreatyPath(site.getUpyunId(),pathInfo.getRelativePath());
		UpYunClient c=UpyunSiteManager.getUpYunClientByUpyunId(site.getUpyunId());
		String url=FileStorageHelper.getUrl(c.getDomain(), pathInfo.getRelativePath());
		YunPathInfo pi=new YunPathInfo(url, relativePath);
		return pi;
	}

	@Override
	public String realPathBackup(String path) {
		String upyunId=FileStorageHelper.getYunId(path);
		String urlpath=FileStorageHelper.getPath(path);
		UpYunClient c=UpyunSiteManager.getUpYunClientByUpyunId(upyunId);
		String bk=c.getDomainBackup();
		if(StringUtils.isNotBlank(bk)) {
			return FileStorageHelper.getUrl(bk, urlpath);
		}
		return FileStorageHelper.getUrl(c.getDomain(), urlpath);
	}
	
	private String getTreatyPath(String yunId,String relativePath) {
		if(relativePath.startsWith("/")) {
			relativePath=relativePath.substring(1);
		}
		String path=FileStorageType.UPYUN.name().toLowerCase()+"-"+yunId+"://"+relativePath;
		return path;
	}

}
